# -*- coding: utf8 -*-
"""
.. module:: burpui.misc.backend.burp.constant
    :platform: Unix
    :synopsis: Burp-UI burp constant module.

.. moduleauthor:: Ziirish <hi+burpui@ziirish.me>

"""

BURP_MINIMAL_VERSION = "burp-2.0.18"
BURP_LIST_BATCH = "2.0.48"
BURP_REVERSE_COUNTERS = "2.1.6"
BURP_STATUS_FORMAT_V2 = BURP_BIND_MULTIPLE = "2.1.10"
BURP_STATUS_DELIMITER = BURP_LISTEN_OPTION = "2.2.10"
