"""initial

Revision ID: 225d9b2f0fb1
Revises: None
Create Date: 2016-07-18 11:22:27.613921

"""

# revision identifiers, used by Alembic.
revision = "225d9b2f0fb1"
down_revision = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "task",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("uuid", sa.String(length=256), nullable=True),
        sa.Column("task", sa.String(length=256), nullable=True),
        sa.Column("user", sa.String(length=256), nullable=True),
        sa.Column("timestamp", sa.DateTime(), nullable=True),
        sa.Column("expire", sa.DateTime(), nullable=True),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("uuid"),
    )
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("task")
    ### end Alembic commands ###
