
          <div id="navbar-config">
            <h4>{{ _('User settings navigation') }}</h4>
            <ul class="nav nav-sidebar bui-scrollspy">
              <li data-target="#sessions"><a class="scroll" href="#sessions"><i class="fa fa-fw fa-list-alt" aria-hidden="true"></i>&nbsp;{{ _('Sessions') }}</a></li>
              <li data-target="#prefs"><a class="scroll" href="#prefs"><i class="fa fa-fw fa-cog" aria-hidden="true"></i>&nbsp;{{ _('Preferences') }}</a></li>
              <li data-target="#hidden"><a class="scroll" href="#hidden"><i class="fa fa-fw fa-eye-slash" aria-hidden="true"></i>&nbsp;{{ _('Hidden') }}</a></li>
              <li data-target="#acl"><a class="scroll" href="#acl"><i class="fa fa-fw fa-lock" aria-hidden="true"></i>&nbsp;{{ _('Authorizations') }}</a></li>
            </ul>
          </div>
