{% extends "layout.html" %}
{% block body %}
        {% include "notifications.html" %}
        <div class="col-sm-9 col-sm-offset-3 col-md-10 col-md-offset-2 main">
          {% include "small_topbar.html" %}
          <ul class="breadcrumb" style="margin-bottom: 5px;">
            {% if server -%}
            <li><a href="{{ url_for('view.home') }}">{{ _('Home') }}</a></li>
            <li class="active">{{ _('Calendar') }}</li>
            {% else -%}
            <li><a href="{{ url_for('view.home') }}">{{ _('Home') }}</a></li>
            <li class="active">{{ _('Calendar') }}</li>
            {% endif -%}
          </ul>
          <br />
          <h1 class="page-header">
            {% if server and client %}
            {{ _('Calendar of %(client)s on %(server)s', client=cname, server=server) }}
            {% elif client %}
            {{ _('Calendar of %(client)s', client=cname) }}
            {% elif server %}
            {{ _('Calendars of %(server)s', server=server) }}
            {% else %}
            {{ _('Calendars') }}
            {% endif %}
          </h1>
          <div id='calendar'></div>
        </div>
{% endblock %}
