
          <div id="navbar-config">
            <h4>{{ _('Administration navigation') }}</h4>
            <ul class="nav nav-sidebar">
              <li {% if authentications or sessions or authentication %}class="active"{% endif %}><a href="{{ url_for('view.admin_authentications') }}"><i class="fa fa-fw fa-id-card-o" aria-hidden="true"></i>&nbsp;{{ _('Authentications') }}</a></li>
              <li {% if authorizations or authorization %}class="active"{% endif %}><a href="{{ url_for('view.admin_authorizations') }}"><i class="fa fa-fw fa-lock" aria-hidden="true"></i>&nbsp;{{ _('Authorizations') }}</a></li>
              <li {% if backends or onebackend %}class="active"{% endif %}><a href="{{ url_for('view.admin_backends') }}"><i class="fa fa-fw fa-database" aria-hidden="true"></i>&nbsp;{{ _('Backends') }}</a></li>
            </ul>
          </div>
