          {% if not cname -%}
          {% set cname = None %}
          {% endif -%}
          {% if not server -%}
          {% set server = None %}
          {% endif -%}
          {% if not settings and not me and not admin -%}
          <div class="btn-group btn-group-justified  hidden-md hidden-lg">
            {% if backup -%}
            <a class="btn btn-default {% if overview %}active{% endif %}" href="{{ url_for('view.client_browse', name=cname, server=server, backup=nbackup) }}"><i class="fa fa-th" aria-hidden="true"></i>&nbsp;{{ _('Overview') }}</a>
            {% elif client -%}
            <a class="btn btn-default {% if overview %}active{% endif %}" href="{{ url_for('view.client', name=cname, server=server) }}"><i class="fa fa-th" aria-hidden="true"></i>&nbsp;{{ _('Overview') }}</a>
            {% elif clients -%}
            <a class="btn btn-default {% if overview %}active{% endif %}" href="{{ url_for('view.clients', server=server) }}"><i class="fa fa-th" aria-hidden="true"></i>&nbsp;{{ _('Overview') }}</a>
            {% elif servers -%}
            <a class="btn btn-default {% if overview %}active{% endif %}" href="{{ url_for('view.servers') }}"><i class="fa fa-th" aria-hidden="true"></i>&nbsp;{{ _('Overview') }}</a>
            {% else -%}
            <a class="btn btn-default {% if overview %}active{% endif %}" href="#"><i class="fa fa-th" aria-hidden="true"></i>&nbsp;{{ _('Overview') }}</a>
            {% endif -%}
            {% if client and not backup -%}
            <a class="btn btn-default {% if report %}active{% endif %}" href="{{ url_for('view.client_report', name=cname, server=server) }}"><i class="fa fa-bar-chart" aria-hidden="true"></i>&nbsp;{{ _('Reports') }}</a>
            {% elif clients -%}
            <a class="btn btn-default {% if report %}active{% endif %}" href="{{ url_for('view.clients_report', server=server) }}"><i class="fa fa-bar-chart" aria-hidden="true"></i>&nbsp;{{ _('Reports') }}</a>
            {% elif servers -%}
            <a class="btn btn-default {% if report %}active{% endif %}" href="{{ url_for('view.servers_report') }}"><i class="fa fa-bar-chart" aria-hidden="true"></i>&nbsp;{{ _('Reports') }}</a>
            {% elif backup -%}
            <a class="btn btn-default {% if report %}active{% endif %}" href="{{ url_for('view.backup_report', name=cname, backup=nbackup, server=server) }}"><i class="fa fa-bar-chart" aria-hidden="true"></i>&nbsp;{{ _('Reports') }}</a>
            {% else -%}
            <a class="btn btn-default {% if report %}active{% endif %}" href="#"><i class="fa fa-bar-chart" aria-hidden="true"></i>&nbsp;{{ _('Reports') }}</a>
            {% endif -%}
            <a class="btn btn-default {% if calendar %}active{% endif %}" href="{{ url_for('view.calendar', client=cname, server=server) }}"><i class="fa fa-calendar" aria-hidden="true"></i>&nbsp;{{ _('Calendar') }}</a>
          </div>
          <br />
          {% endif -%}
          {% if admin -%}
          <div class="btn-group btn-group-justified  hidden-md hidden-lg">
            <a class="btn btn-default {% if authentications %}active{% endif %}" href="{{ url_for('view.admin_authentications') }}"><i class="fa fa-fw fa-id-card-o" aria-hidden="true"></i>&nbsp;{{ _('Authentication') }}</a>
            <a class="btn btn-default {% if authorizations %}active{% endif %}" href="{{ url_for('view.admin_authorizations') }}"><i class="fa fa-fw fa-lock" aria-hidden="true"></i>&nbsp;{{ _('Authorization') }}</a>
            <a class="btn btn-default {% if backends or onebackend %}active{% endif %}" href="{{ url_for('view.admin_backends') }}"><i class="fa fa-fw fa-database" aria-hidden="true"></i>&nbsp;{{ _('Backends') }}</a>
          </div>
          <br />
          {% endif -%}
