"""increase pref value size

Revision ID: 695dcbd29d4f
Revises: 56de018f4d88
Create Date: 2018-10-02 13:56:57.660204

"""

# revision identifiers, used by Alembic.
revision = "695dcbd29d4f"
down_revision = "56de018f4d88"

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("pref", schema=None) as batch_op:
        batch_op.alter_column(
            "value",
            existing_type=sa.VARCHAR(length=256),
            type_=sa.Text(),
            existing_nullable=True,
        )

    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("pref", schema=None) as batch_op:
        batch_op.alter_column(
            "value",
            existing_type=sa.Text(),
            type_=sa.VARCHAR(length=256),
            existing_nullable=True,
        )

    ### end Alembic commands ###
